@@ -38,7 +38,7 @@ use CGI::FormBuilder;
 use HTML::Entities;
 use URI::Escape;
 
-our $VERSION = '1.00_02';
+our $VERSION = '1.10';
 
 =encoding utf8
 
@@ -111,6 +111,7 @@ connection.
         display_columns  => [ qw( id venue_id name division ) ],
         deleteable => 1,
         editable => 1,
+        addable => 0,   # does not allow adding rows
         sortable => 1,
         paginate => 300,
         template => 'simple_crud.tt',
@@ -370,7 +371,12 @@ given, and the edit form will have a "Delete $record_title" button.
 =item C<editable>
 
 Specify whether to support editing records.  Defaults to true.  If set to a
-false value, it will not be possible to add or edit rows in the table.
+false value, it will not be possible to add or edit rows in the table. See also C<addable>.
+
+=item C<addable>
+
+Specify whether to support adding records.  Defaults to the value of C<editable> if set,
+or true otherwise.  If set to a false value, it will not be possible to add rows in the table.
 
 =item C<sortable>
 
@@ -430,6 +436,11 @@ build the custom column from, and C<transform>, a subref to be used as a
 HTML::Table::FromDatabase callback on the resulting column.  If no
 C<transform> is provided, sub { return shift; } will be used.
 
+If your custom column has the same name as an existing column, your customizations
+will be used in-place to override the display of the content in that column.
+If sorting is enabled, the column will be sorted by the underlying database content 
+for that row, and not by the output of your transform function.
+
 For a somewhat spurious example:
 
     ...
@@ -540,6 +551,7 @@ sub simple_crud {
     $args{key_column}   ||= 'id';
     $args{record_title} ||= 'record';
     $args{editable}         = 1 unless exists $args{editable};
+    $args{addable}          = $args{editable} unless exists $args{addable};
     $args{query_auto_focus} = 1 unless exists $args{query_auto_focus};
 
     # Sanitise things we'll have to interpolate into queries (yes, that makes me
@@ -558,7 +570,15 @@ sub simple_crud {
 
     if ($args{editable}) {
         _ensure_auth('edit', $handler, \%args);
-        for ('/add', '/edit/:id') {
+        for ('/edit/:id') {
+            my $url = _construct_url($args{dancer_prefix}, $args{prefix}, $_);
+            Dancer::Logger::debug("Setting up route for $url");
+            any ['get', 'post'] => $url => $handler;
+        }
+    }
+    if ($args{addable}) {
+        _ensure_auth('edit', $handler, \%args);
+        for ('/add') {
             my $url = _construct_url($args{dancer_prefix}, $args{prefix}, $_);
             Dancer::Logger::debug("Setting up route for $url");
             any ['get', 'post'] => $url => $handler;
@@ -610,10 +630,27 @@ CONFIRMDELETE
             my $dbh = database($args{db_connection_name});
             my $where = _get_where_filter_from_args(\%args);
             $where->{$key_column} = $id;
-            $dbh->quick_delete($table_name, $where)
+
+            my %params = params;
+            my $meta_for_hook = {
+                args => \%args,
+                params => \%params,
+                table_name => $table_name,
+                key_column => $key_column,
+            };
+            # fire the pre-delete hook, in case user wants to perform trickery before the delete
+            execute_hook('delete_row_pre_delete', $meta_for_hook );
+
+            my $rows_deleted = $dbh->quick_delete($table_name, $where)
                 or return _apply_template("<p>Failed to delete!</p>",
                 $args{'template'});
 
+            $meta_for_hook->{success} = $rows_deleted;
+            if ($rows_deleted) {
+                # post-delete hook, in case user wants to do cunning things after the delete
+                execute_hook('delete_row_post_delete', $meta_for_hook );
+            }
+
             redirect _external_url($args{dancer_prefix}, $args{prefix});
         };
         _ensure_auth('edit', $delete_handler, \%args);
@@ -665,6 +702,8 @@ register_hook(qw(
     add_edit_row
     add_edit_row_pre_save
     add_edit_row_post_save
+    delete_row_pre_delete
+    delete_row_post_delete
 ));
 register_plugin;
 
@@ -898,7 +937,7 @@ sub _create_add_edit_route {
             # against it if they need to.  last_insert_id in some instances requires
             # catalog, schema, etc args, so we can't just call it and save the result.
             # important that we don't do any more database operations that would change
-            # last_insert_id between here and the hook, or this won'w work.
+            # last_insert_id between here and the hook, or this won't work.
             $meta_for_hook->{dbh} = $dbh;
             $verb = 'create new';
         }
@@ -979,6 +1018,7 @@ sub _create_list_handler {
     my @searchtypes = (
         [ e   => { name => "Equals",           cmp => "=" } ],
         [ c   => { name => "Contains",         cmp => "like" } ],
+        [ b   => { name => "Begins With",      cmp => "like" } ],
         [ ne  => { name => "Does Not Equal",   cmp => "!=" } ],
         [ nc  => { name => "Does Not Contain", cmp => "not like" } ],
 
@@ -1070,17 +1110,20 @@ SEARCHFORM
         $args->{custom_columns} = \@custom_cols_list;
     }
 
+    # If we're not overriding a column with the same name, then add custom column
     for my $custom_col_spec (@{ $args->{custom_columns} || [] }) {
         my $column_alias = $custom_col_spec->{name};
-        my $raw_column = $custom_col_spec->{raw_column}
-            or die "you must specify a raw_column that "
-                 . "$column_alias will be built using";
-        if ($raw_column =~ /^[\w_]+$/) {
-            push @custom_cols, "$table_name." 
-                . $dbh->quote_identifier($raw_column) 
-                . " AS ". $dbh->quote_identifier($column_alias);
-        } else {
-            push @custom_cols, "$raw_column AS $column_alias";
+        if( ! grep { $column_alias eq $_ } @select_cols) {
+            my $raw_column = $custom_col_spec->{raw_column}
+                or die "you must specify a raw_column that "
+                     . "$column_alias will be built using";
+            if ($raw_column =~ /^[\w_]+$/) {
+                push @custom_cols, "$table_name." 
+                    . $dbh->quote_identifier($raw_column) 
+                    . " AS ". $dbh->quote_identifier($column_alias);
+            } else {
+                push @custom_cols, "$raw_column AS $column_alias";
+            }
         }
     }
 
@@ -1145,6 +1188,8 @@ SEARCHFORM
                 my $search_value = $q;
                 if ($st eq 'c' || $st eq 'nc') {
                     $search_value = '%' . $search_value . '%';
+                } elsif ($st eq 'b') {
+                    $search_value = $search_value . '%';
                 }
 
                 my ($searchtype_row) = grep { $_->[0] eq $st } @searchtypes;
@@ -1155,9 +1200,7 @@ SEARCHFORM
                     . " $cmp ?" );
                 push(@search_binds, $search_value);
 
-                my $matchtype = $st eq "c" ? "contains": 
-                                $st eq "nc" ? "does not contain" :
-                                $st eq "ne" ? "does not equal": "equals";
+                my $matchtype = lc($searchtype_row->[1]->{name} || "equals");
                 $html
                     .= sprintf(
                     "<p>Showing results from searching for '%s' %s '%s'",
@@ -1234,7 +1277,8 @@ SEARCHFORM
         } @all_cols;
 
         # And for custom columns, do the prettification, but don't include a
-        # link for sorting - as we can't sort by them currently (the sorting is
+        # link for sorting unless we're overriding the display of an already 
+        # sortable column. We can't sort non-overridden custom columns (sorting is
         # done by SQL, and the custom column values are calculated after we get
         # the results from the SQL query, so to support sorting by them we'd
         # have to stop getting the database to sort the data and sort it
@@ -1243,8 +1287,10 @@ SEARCHFORM
             for my $custom_column_name (
                 map { $_->{name} } @{ $args->{custom_columns} }
             ) {
-                $columns_sort_options{$custom_column_name}
-                    = _prettify_column_name($custom_column_name);
+                if ( !grep { $_ eq $custom_column_name } @all_cols) {
+                    $columns_sort_options{$custom_column_name}
+                        = _prettify_column_name($custom_column_name);
+                }
             }
         }
 
@@ -1362,7 +1408,7 @@ SEARCHFORM
 
     $html .= $table->getTable || '';
 
-    if ($args->{editable} && _has_permission('edit', $args)) {
+    if ($args->{addable} && _has_permission('edit', $args)) {
         $html .= sprintf '<a href="%s">Add a new %s</a></p>',
             _external_url($args->{dancer_prefix}, $args->{prefix}, '/add'),
             $args->{record_title};
@@ -1670,7 +1716,9 @@ args from the original route setup (C<args>), the table's key column
 (C<key_column>), and the values of the editable params (C<params>).
 
 In the post-save hook, you are also sent C<success> (the return value of
-quick_insert or quick_update) telling you if the save was successful,
+quick_insert or quick_update) telling you if the save was successful
+(which is a little redundant because your post-save hook won't be called unless
+the insert or update was successful). You'll also get
 C<dbh> giving you the instance of the handle used to save the entity
 (so you can access last_insert_id()), and C<verb> (currently either
 'create new' or 'update').
@@ -1686,6 +1734,19 @@ of example that you have a C<hash_pw()> function to return a hashed password:
       }
   };
 
+=head2 delete_row_pre_delete, delete_row_post_delete
+
+These fire right before and after a row is deleted.  As with the
+add_edit_row_pre_save and add_edit_row_post_save hooks, these are
+passed a hashref with metadata such as the name of the table
+(in C<table_name>), the args from the original route setup (C<args>),
+the table's key column (C<key_column>), and the values of the
+editable params (C<params>). As with the post-save hook, delete_row_post_delete hook won't be
+called if we weren't able to delete the row.
+
+You could use these to clean up ancillary data associated with a
+database row when it was deleted, for example.
+
 =head1 AUTHOR
 
 David Precious, C<< <davidp@preshweb.co.uk> >>
