@@ -32,6 +32,12 @@ my $custom_column = { name => 'extra', raw_column => 'id', transform => sub { "H
 # now set up our simple_crud interfaci
 simple_crud( prefix => '/users'  ,              record_title=>'A', db_table => 'users', editable => 0, );
 simple_crud( prefix => '/users_editable',       record_title=>'A', db_table => 'users', editable => 1, );
+simple_crud( prefix => '/users_editable_not_addable',       
+                                                record_title=>'A', db_table => 'users', editable => 1, addable => 0);
 simple_crud( prefix => '/users_custom_columns', record_title=>'A', db_table => 'users', editable => 0, custom_columns => [ $custom_column ] );
 
+# override display of 'username' column
+simple_crud( prefix => '/users_customized_column', record_title=>'A', db_table => 'users', editable => 0, sortable=>1,
+                custom_columns => [ { name => "username", raw_column=>"username", transform => sub { "Username: $_[0]" } } ] );
+
 1;
